@@ -1,41 +1,92 @@
 /*
- * This file contains the dasdinfo tool which can be used
- * to get a unique DASD ID.
+ * dasdinfo - Display unique DASD ID, either UID or volser
  *
- * Copyright IBM Corp. 2007
+ * Copyright IBM Corp. 2007, 2017
  *
- * Author(s): Volker Sameske (sameske@de.ibm.com)
+ * s390-tools is free software; you can redistribute it and/or modify
+ * it under the terms of the MIT license. See LICENSE for details.
  */
 
-#define _GNU_SOURCE
-
-#include <stdlib.h>
-#include <stdio.h>
-#include <getopt.h>
-#include <string.h>
-#include <fcntl.h>
-#include <unistd.h>
-#include <errno.h>
 #include <dirent.h>
+#include <errno.h>
+#include <fcntl.h>
 #include <ftw.h>
-#include <sys/utsname.h>
-#include <sys/types.h>
-#include <sys/stat.h>
+#include <getopt.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 #include <sys/ioctl.h>
+#include <sys/stat.h>
 #include <sys/sysmacros.h>
-#include "zt_common.h"
+#include <sys/types.h>
+#include <sys/utsname.h>
+#include <unistd.h>
 
-#define READCHUNK 80
-#define BLKSSZGET    _IO(0x12,104)
-#define DASD_IOCTL_LETTER 'D'
-#define BIODASDINFO  _IOR(DASD_IOCTL_LETTER,1,struct dasd_information)
-#define BIODASDINFO2 _IOR(DASD_IOCTL_LETTER,3,struct dasd_information2)
+#include "lib/dasd_base.h"
+#include "lib/util_base.h"
+#include "lib/util_file.h"
+#include "lib/util_libc.h"
+#include "lib/util_opt.h"
+#include "lib/util_prg.h"
+#include "lib/zt_common.h"
+
+#define RD_BUFFER_SIZE 80
 #define TEMP_DEV_MAX_RETRIES    1000
 
-#define MAX(x,y) ((x)<(y)?(y):(x))
+static const struct util_prg prg = {
+	.desc = "Display DASD volume serial number and ID information",
+	.args = "-i BUSID | -b BLOCKDEV | -d DEVNODE",
+	.copyright_vec = {
+		{
+			.owner = "IBM Corp.",
+			.pub_first = 2007,
+		},
+		UTIL_PRG_COPYRIGHT_END
+	}
+};
 
-static const char tool_name[] = "dasdinfo: zSeries DASD information program";
-static const char copyright_notice[] = "Copyright IBM Corp. 2007";
+static struct util_opt opt_vec[] = {
+	UTIL_OPT_SECTION("DEVICE"),
+	{
+		.option = { "block", required_argument, NULL, 'b' },
+		.argument = "BLOCKDEV",
+		.desc = "Block device name, e.g. dasdb",
+	},
+	{
+		.option = { "devnode", required_argument, NULL, 'd' },
+		.argument = "DEVNODE",
+		.desc = "Device node, e.g. /dev/dasda",
+	},
+	{
+		.option = { "busid", required_argument, NULL, 'i' },
+		.argument = "BUSID",
+		.desc = "Bus ID, e.g. 0.0.e910",
+	},
+	UTIL_OPT_SECTION("OPTIONS"),
+	{
+		.option = { "label", no_argument, NULL, 'l' },
+		.desc = "Print DASD volume label (volser)",
+	},
+	{
+		.option = { "uid", no_argument, NULL, 'u' },
+		.desc = "Print DASD uid (without z/VM minidisk token)",
+	},
+	{
+		.option = { "extended-uid", no_argument, NULL, 'x' },
+		.desc = "Print DASD uid (including z/VM minidisk token)",
+	},
+	{
+		.option = { "all", no_argument, NULL, 'a' },
+		.desc = "Same as -u -x -l",
+	},
+	{
+		.option = { "export", no_argument, NULL, 'e' },
+		.desc = "Export ID_BUS, ID_TYPE, ID_SERIAL for use in udev",
+	},
+	UTIL_OPT_HELP,
+	UTIL_OPT_VERSION,
+	UTIL_OPT_END
+};
 
 /* needed because ftw can not pass arbitrary arguments */
 static char *searchbusid;
@@ -47,108 +98,7 @@ struct volume_label {
 	char volid[6];
 } __attribute__ ((packed));
 
-struct dasd_information2 {
-	unsigned int devno;
-	unsigned int real_devno;
-	unsigned int schid;
-	unsigned int cu_type  : 16;
-	unsigned int cu_model :  8;
-	unsigned int dev_type : 16;
-	unsigned int dev_model : 8;
-	unsigned int open_count;
-	unsigned int req_queue_len;
-	unsigned int chanq_len;
-	char type[4];
-	unsigned int status;
-	unsigned int label_block;
-	unsigned int FBA_layout;
-	unsigned int characteristics_size;
-	unsigned int confdata_size;
-	char characteristics[64];
-	char configuration_data[256];
-	unsigned int format;
-	unsigned int features;
-	unsigned int reserved0;
-	unsigned int reserved1;
-	unsigned int reserved2;
-	unsigned int reserved3;
-	unsigned int reserved4;
-	unsigned int reserved5;
-	unsigned int reserved6;
-	unsigned int reserved7;
-};
-
-struct dasd_information {
-	unsigned int devno;
-	unsigned int real_devno;
-	unsigned int schid;
-	unsigned int cu_type  : 16;
-	unsigned int cu_model :  8;
-	unsigned int dev_type : 16;
-	unsigned int dev_model : 8;
-	unsigned int open_count;
-	unsigned int req_queue_len;
-	unsigned int chanq_len;
-	char type[4];
-	unsigned int status;
-	unsigned int label_block;
-	unsigned int FBA_layout;
-	unsigned int characteristics_size;
-	unsigned int confdata_size;
-	char characteristics[64];
-	char configuration_data[256];
-};
-
-struct dasd_data
-{
-	struct dasd_information2 dasd_info;
-	int dasd_info_version;
-	int blksize;
-};
-
-static void dinfo_print_version (void)
-{
-	printf ("%s version %s\n", tool_name, RELEASE_STRING);
-	printf ("%s\n", copyright_notice);
-}
-
-static void dinfo_print_usage(char *cmd)
-{
-	printf(""
-	       "Display specific information about a specified DASD device.\n"
-	       "\n"
-	       "Usage: %s [-a] [-u] [-x] [-l] [-e]\n"
-	       "                {-i <busid> | -b <blockdev> | -d <devnode>}\n"
-	       "       %s [-h] [-v]\n"
-	       "\n"
-	       "where:\n"
-	       "    -a|--all\n"
-	       "             same as -u -x -l\n"
-	       "    -u|--uid\n"
-	       "             print DASD uid (without z/VM minidisk token)\n"
-	       "    -x|--extended-uid\n"
-	       "             print DASD uid (including z/VM minidisk token)\n"
-	       "    -l|--label\n"
-	       "             print DASD volume label (volser)\n"
-	       "    -i|--busid <busid>\n"
-	       "             bus ID, e.g. 0.0.e910\n"
-	       "    -b|--block <blockdev>\n"
-	       "             block device name, e.g. dasdb\n"
-	       "    -d|--devnode <devnode>\n"
-	       "             device node, e.g. /dev/dasda\n"
-	       "    -e|--export\n"
-	       "             print all values (ID_BUS, ID_TYPE, ID_SERIAL)\n"
-	       "    -h|--help\n"
-	       "             prints this usage text\n"
-	       "    -v|--version\n"
-	       "             prints the version number\n"
-	       "\n"
-	       "Example: %s -u -b dasda\n",
-	       cmd,cmd,cmd);
-}
-
-static char EBCtoASC[256] =
-{
+static char EBCtoASC[256] = {
 /* 0x00  NUL   SOH   STX   ETX  *SEL    HT  *RNL   DEL */
 	0x00, 0x01, 0x02, 0x03, 0x07, 0x09, 0x07, 0x7F,
 /* 0x08  -GE  -SPS  -RPT    VT    FF    CR    SO    SI */
@@ -215,86 +165,64 @@ static char EBCtoASC[256] =
 	0x38, 0x39, 0x07, 0x07, 0x9A, 0x07, 0x07, 0x07
 };
 
-static char *dinfo_ebcdic_dec (char *source, char *target, int l)
+static char *dinfo_ebcdic_dec(char *source, char *target, int l)
 {
 	int i;
 
 	for (i = 0; i < l; i++)
-		target[i]=EBCtoASC[(unsigned char)(source[i])];
+		target[i] = EBCtoASC[(unsigned char)(source[i])];
 
 	return target;
 }
 
-static int dinfo_read_dasd_uid (char *uidfile, char *readbuf, int readbuflen)
+static int dinfo_read_dasd_uid(char *uidfile, char *readbuf)
 {
-	FILE *dasduid;
-	int offset = 0;
-
-	if ((dasduid = fopen(uidfile,"r")) == NULL)
-		return -1;
-
-	while (fgets(readbuf + offset, READCHUNK, dasduid)  &&
-	       readbuf[strlen(readbuf)-1] != '\n' ) {
-		offset += READCHUNK-1;
-		if ( offset+READCHUNK-1 >= readbuflen ) {
-			readbuf = realloc(readbuf,
-					  readbuflen + READCHUNK-1);
-			readbuflen += READCHUNK-1;
-		}
-	}
-	fclose(dasduid);
-
-	if (strlen(readbuf) <= 1)
-		return -1;
-
-	return 0;
+	return util_file_read_line(readbuf, RD_BUFFER_SIZE, uidfile);
 }
 
-static int dinfo_read_dasd_vlabel (char *device, struct volume_label *vlabel,
-				   char *readbuf)
+static int dinfo_read_dasd_vlabel(char *device, struct volume_label *vlabel,
+				  char *readbuf)
 {
-	struct dasd_data data;
 	struct volume_label tmp;
-	unsigned int vlsize = sizeof(struct volume_label);
+	int vlsize = sizeof(struct volume_label);
+	dasd_information2_t dasd_info;
 	unsigned long vlabel_start;
+	unsigned int blksize;
 	char vollbl[5];
 	int f;
 	char *space;
 
-	if ((f = open(device, O_RDONLY)) < 0) {
-		printf("Could not open device node.\n");
+	if (dasd_get_blocksize(device, &blksize) != 0) {
+		warnx("Unable to figure out block size");
 		goto error;
 	}
 
-	if (ioctl(f, BLKSSZGET, &data.blksize) != 0) {
-		printf("Unable to figure out block size.\n");
+	if (dasd_get_info(device, &dasd_info) != 0) {
+		warnx("Unable to figure out DASD information");
 		goto error;
 	}
 
-	if (ioctl(f, BIODASDINFO2, &data.dasd_info) == 0)
-		data.dasd_info_version = 2;
-	else {
-		if (ioctl(f, BIODASDINFO, &data.dasd_info) != 0) {
-			printf("Unable to figure out DASD informations.\n");
-			goto error;
-		}
+	f = open(device, O_RDONLY);
+	if (f < 0) {
+		warnx("Could not open device node");
+		goto error;
 	}
 
-	vlabel_start = data.dasd_info.label_block * data.blksize;
+	vlabel_start = dasd_info.label_block * blksize;
 	if (lseek(f, vlabel_start, SEEK_SET) < 0)
-		goto error;
+		goto error_close;
 
 	bzero(vlabel, vlsize);
 
 	if (read(f, vlabel, vlsize) != vlsize) {
-		printf("Could not read volume label.\n");
-		goto error;
+		warnx("Could not read volume label");
+		goto error_close;
 	}
 
-	if (data.dasd_info.FBA_layout) {
+	if (dasd_info.FBA_layout) {
 		bzero(&tmp, vlsize);
 		memcpy(&tmp, vlabel, vlsize);
-		memcpy(vlabel->vollbl, &tmp, vlsize-4);
+		memcpy(vlabel->vollbl, &tmp, vlsize - 4);
 	}
 
 	close(f);
@@ -309,16 +237,17 @@ static int dinfo_read_dasd_vlabel (char *device, struct volume_label *vlabel,
 	    (strncmp(vollbl, "CMS1", 4) == 0)) {
 		strncpy(readbuf, vlabel->volid, 6);
 		dinfo_ebcdic_dec(readbuf, readbuf, 6);
-		space = strchr(readbuf,' ');
+		space = strchr(readbuf, ' ');
 		if (space)
 			*space = 0;
-	} else
+	} else {
 		strcpy(readbuf, "");
-
+	}
 
 	return 0;
-error:
+error_close:
 	close(f);
+error:
 	return -1;
 }
 
@@ -327,10 +256,9 @@ static void *dinfo_malloc(size_t size)
 	void *result;
 
 	result = malloc(size);
-	if (result == NULL) {
-		printf("Could not allocate %lld bytes of memory",
-			(unsigned long long) size);
-	}
+	if (result == NULL)
+		warnx("Could not allocate %lu bytes of memory", size);
+
 	return result;
 }
 
@@ -340,7 +268,7 @@ static char *dinfo_make_path(char *dirname, char *filename)
 	size_t len;
 
 	len = strlen(dirname) + strlen(filename) + 2;
-	result = (char *) dinfo_malloc(len);
+	result = (char *)dinfo_malloc(len);
 	if (result == NULL)
 		return NULL;
 	sprintf(result, "%s/%s", dirname, filename);
@@ -350,8 +278,8 @@ static char *dinfo_make_path(char *dirname, char *filename)
 static int dinfo_create_devnode(dev_t dev, char **devno)
 {
 	char *result;
-	char * pathname[] = { "/dev", getenv("TMPDIR"), "/tmp",
-				getenv("HOME"), "." , "/"};
+	char *pathname[] = { "/dev", getenv("TMPDIR"), "/tmp",
+		getenv("HOME"), ".", "/"};
 	char filename[] = "dasdinfo0000";
 	mode_t mode;
 	unsigned int path;
@@ -362,10 +290,10 @@ static int dinfo_create_devnode(dev_t dev, char **devno)
 	mode = S_IFBLK | S_IRWXU;
 
 	/* Try several locations for the temporary device node. */
-	for (path=0; path < sizeof(pathname) / sizeof(pathname[0]); path++) {
+	for (path = 0; path < UTIL_ARRAY_SIZE(pathname); path++) {
 		if (pathname[path] == NULL)
 			continue;
-		for (retry=0; retry < TEMP_DEV_MAX_RETRIES; retry++) {
+		for (retry = 0; retry < TEMP_DEV_MAX_RETRIES; retry++) {
 			sprintf(filename, "dasdinfo%04d", retry);
 			result = dinfo_make_path(pathname[path], filename);
 			if (result == NULL)
@@ -374,8 +302,9 @@ static int dinfo_create_devnode(dev_t dev, char **devno)
 			if (rc == 0) {
 				/* Need this test to cover
 				 * 'nodev'-mounted
-				 * filesystems. */
-				fd = open(result, O_RDWR);
+				 * filesystems.
+				 */
+				fd = open(result, O_RDONLY);
 				if (fd != -1) {
 					close(fd);
 					*devno = result;
@@ -383,33 +312,33 @@ static int dinfo_create_devnode(dev_t dev, char **devno)
 				}
 				remove(result);
 				retry = TEMP_DEV_MAX_RETRIES;
-			} else if (errno != EEXIST)
+			} else if (errno != EEXIST) {
 				retry = TEMP_DEV_MAX_RETRIES;
+			}
 			free(result);
 		}
 	}
-	printf("Error: Unable to create temporary device node");
+	warnx("Error: Unable to create temporary device node");
 	return -1;
 }
 
 static void dinfo_free_devnode(char *device)
 {
-	if (remove(device)) {
-		printf("Warning: Could not remove "
-			"temporary file %s", device);
-	}
+	if (remove(device))
+		warnx("Warning: Could not remove temporary file %s", device);
 }
 
-static int dinfo_extract_dev(dev_t *dev, char *str, int readbuflen)
+static int dinfo_extract_dev(dev_t *dev, char *str)
 {
-	char tmp[readbuflen];
+	char tmp[RD_BUFFER_SIZE];
 	char *p = NULL;
 	int ma, mi;
 
-	bzero(tmp, readbuflen);
-	strncpy(tmp, str, readbuflen);
-	if ((p = strchr(tmp, ':')) == NULL) {
-		printf("Error: unable to extract major/minor\n");
+	bzero(tmp, RD_BUFFER_SIZE);
+	util_strlcpy(tmp, str, RD_BUFFER_SIZE);
+	p = strchr(tmp, ':');
+	if (p == NULL) {
+		warnx("Error: unable to extract major/minor");
 		return -1;
 	}
 
@@ -424,49 +353,30 @@ static int dinfo_extract_dev(dev_t *dev, char *str, int readbuflen)
 
 static int dinfo_get_dev_from_blockdev(char *blockdev, dev_t *dev)
 {
-	FILE *dasddev;
-	int offset = 0;
-	char *devfile = NULL;
 	char *readbuf = NULL;
-	int readbuflen = READCHUNK;
 
-	if ((devfile = dinfo_malloc(readbuflen)) == NULL)
-		return -1;
-
-	sprintf(devfile,"/sys/block/%s/dev", blockdev);
-
-	if ((readbuf = dinfo_malloc(readbuflen)) == NULL) {
-		printf("Error: Not enough memory to allocate readbuffer\n");
+	readbuf = dinfo_malloc(RD_BUFFER_SIZE);
+	if (!readbuf) {
+		warnx("Error: Not enough memory to allocate readbuffer");
 		return -1;
 	}
-
-	if ((dasddev = fopen(devfile,"r")) == NULL)
+	if (util_file_read_line(readbuf, RD_BUFFER_SIZE,
+				"/sys/block/%s/dev", blockdev) < 0)
 		return -1;
-
-	while (fgets(readbuf + offset, READCHUNK, dasddev)  &&
-	       readbuf[strlen(readbuf)-1] != '\n' ) {
-		offset += READCHUNK-1;
-		if (offset+READCHUNK-1 >= readbuflen) {
-			readbuf = realloc(readbuf,
-					  readbuflen + READCHUNK-1);
-			readbuflen += READCHUNK-1;
-		}
-	}
-	fclose(dasddev);
-
-	if (dinfo_extract_dev(dev, readbuf, readbuflen) != 0)
+	if (dinfo_extract_dev(dev, readbuf) != 0)
 		return -1;
 
 	return 0;
 }
 
 static int
-dinfo_is_busiddir(const char *fpath, const struct stat *sb,
+dinfo_is_busiddir(const char *fpath, const struct stat *UNUSED(sb),
 		  int tflag, struct FTW *ftwbuf)
 {
 	char *tempdir;
 	char linkdir[128];
 	ssize_t i;
+
 	if (tflag != FTW_D || (strncmp((fpath + ftwbuf->base), searchbusid,
 				       strlen(searchbusid)) != 0))
 		return FTW_CONTINUE;
@@ -482,7 +392,7 @@ dinfo_is_busiddir(const char *fpath, const struct stat *sb,
 	if ((i < 0) || (i >= 128))
 		return -1;
 	/* append '\0' because readlink returns non zero terminated string */
-	tempdir[i+1] = '\0';
+	tempdir[i + 1] = '\0';
 	if (strstr(linkdir, "dasd") == NULL)
 		return FTW_CONTINUE;
 	free(busiddir);
@@ -498,15 +408,17 @@ dinfo_find_entry(const char *dir, const char *searchstring,
 {
 	DIR *directory = NULL;
 	struct dirent *dir_entry = NULL;
+
 	directory = opendir(dir);
 	if (directory == NULL)
 		return -1;
 	while ((dir_entry = readdir(directory)) != NULL) {
-		/* compare if the found entry has exactly the same name
-		   and type as searched */
+		/* compare if the found entry has exactly the same name and type
+		 * as searched
+		 */
 		if ((strncmp(dir_entry->d_name, searchstring,
-			     strlen(searchstring)) == 0)
-		    && (dir_entry->d_type & type)) {
+			     strlen(searchstring)) == 0) &&
+		    (dir_entry->d_type & type)) {
 			*result = strdup(dir_entry->d_name);
 			if (*result == NULL)
 				goto out;
@@ -569,55 +481,49 @@ out2:
 static int dinfo_get_uid_from_devnode(char **uidfile, char *devnode)
 {
 	struct stat stat_buffer;
-	char stat_dev[READCHUNK];
-	char sys_dev_path[READCHUNK];
+	char stat_dev[RD_BUFFER_SIZE];
 	char *readbuf;
 	DIR *directory = NULL;
 	struct dirent *dir_entry = NULL;
-	FILE *block_dev;
-	int readbuflen = READCHUNK;
-	int offset;
+	int rc = 0;
 
 	if (stat(devnode, &stat_buffer) != 0) {
-		printf("Error: could not stat %s\n", devnode);
+		warnx("Error: could not stat %s", devnode);
 		return -1;
 	}
 
 	sprintf(stat_dev, "%d:%d", major(stat_buffer.st_rdev),
 		minor(stat_buffer.st_rdev));
 
-	if ((directory = opendir("/sys/block/")) == NULL) {
-		printf("Error: could not open directory /sys/block\n");
+	directory = opendir("/sys/block/");
+	if (directory == NULL) {
+		warnx("Error: could not open directory /sys/block");
 		return -1;
 	}
 
-	if ((readbuf = dinfo_malloc(readbuflen)) == NULL) {
-		printf("Error: Not enough memory to allocate readbuffer\n");
+	readbuf = dinfo_malloc(RD_BUFFER_SIZE);
+	if (!readbuf) {
+		warnx("Error: Not enough memory to allocate readbuffer");
 		return -1;
 	}
 
 	while ((dir_entry = readdir(directory)) != NULL) {
-		sprintf(sys_dev_path, "/sys/block/%s/dev", dir_entry->d_name);
-
-		if ((block_dev = fopen(sys_dev_path,"r")) == NULL)
+		if (util_file_read_line(readbuf, RD_BUFFER_SIZE,
+					"/sys/block/%s/dev",
+					dir_entry->d_name) < 0)
 			continue;
 
-		offset = 0;
-		while (fgets(readbuf + offset, READCHUNK, block_dev)  &&
-		       readbuf[strlen(readbuf)-1] != '\n' ) {
-			offset += READCHUNK-1;
-			if ( offset+READCHUNK-1 >= readbuflen ) {
-				readbuf = realloc(readbuf,
-						  readbuflen + READCHUNK-1);
-				readbuflen += READCHUNK-1;
-			}
-		}
-		fclose(block_dev);
-
 		if (strncmp(stat_dev, readbuf,
-			    MAX(strlen(stat_dev), strlen(readbuf)-1)) == 0) {
-			sprintf(*uidfile,"/sys/block/%s/device/uid",
-				dir_entry->d_name);
+			    MAX(strlen(stat_dev), strlen(readbuf) - 1)) == 0) {
+			rc = snprintf(*uidfile, RD_BUFFER_SIZE,
+				      "/sys/block/%s/device/uid",
+				      dir_entry->d_name);
+			if (rc >= RD_BUFFER_SIZE) {
+				fprintf(stderr,
+					"Error: Device name was truncated\n");
+				return -1;
+			}
+
 			break;
 		}
 	}
@@ -626,14 +532,13 @@ static int dinfo_get_uid_from_devnode(char **uidfile, char *devnode)
 	return 0;
 }
 
-int main(int argc, char * argv[])
+int main(int argc, char *argv[])
 {
 	struct utsname uname_buf;
 	int version, release;
 	char *uidfile = NULL;
 	char *device = NULL;
 	char *readbuf = NULL;
-	int readbuflen = READCHUNK;
 	dev_t dev;
 	int export = 0;
 	int c;
@@ -645,26 +550,13 @@ int main(int argc, char * argv[])
 	char *devnode = NULL;
 	struct volume_label vlabel;
 	char *srchuid;
-	int i;
+	int i, rc = 0;
+
+	util_prg_init(&prg);
+	util_opt_init(opt_vec, NULL);
 
 	while (1) {
-		int option_index = 0;
-		static struct option long_options[] = {
-			{"all",          0, 0, 'a'},
-			{"uid",          0, 0, 'u'},
-			{"extended-uid", 0, 0, 'x'},
-			{"label",        0, 0, 'l'},
-			{"busid",        1, 0, 'i'},
-			{"block",        1, 0, 'b'},
-			{"devnode",      1, 0, 'd'},
-			{"export",       0, 0, 'e'},
-			{"help",         0, 0, 'h'},
-			{"version",      0, 0, 'v'},
-			{0, 0, 0, 0}
-		};
-
-		c = getopt_long (argc, argv, "vhaeuxlb:i:d:",
-				 long_options, &option_index);
+		c = util_opt_getopt_long(argc, argv);
 		if (c == -1)
 			break;
 
@@ -684,23 +576,24 @@ int main(int argc, char * argv[])
 			print_vlabel = 1;
 			break;
 		case 'i':
-			busid=strdup(optarg);
+			busid = strdup(optarg);
 			break;
 		case 'b':
-			blockdev=strdup(optarg);
+			blockdev = strdup(optarg);
 			break;
 		case 'd':
-			devnode=strdup(optarg);
+			devnode = strdup(optarg);
 			break;
 		case 'e':
 			export = 1;
 			break;
 		case 'h':
-			dinfo_print_usage(argv[0]);
-			exit(0);
+			util_prg_print_help();
+			util_opt_print_help();
+			exit(EXIT_SUCCESS);
 		case 'v':
-			dinfo_print_version();
-			exit(0);
+			util_prg_print_version();
+			exit(EXIT_SUCCESS);
 		default:
 			fprintf(stderr, "Try 'dasdinfo --help' for more "
 				"information.\n");
@@ -709,40 +602,39 @@ int main(int argc, char * argv[])
 	}
 
 	uname(&uname_buf);
-	sscanf(uname_buf.release, "%d.%d", &version,&release);
-	if (strcmp(uname_buf.sysname,"Linux") ||
-	    version < 2 || release < 6) {
-		printf("%s %d.%d is not supported\n", uname_buf.sysname,
-			version,release);
-		return -1;
+	sscanf(uname_buf.release, "%d.%d", &version, &release);
+	if (strcmp(uname_buf.sysname, "Linux") ||
+	    version < 2 || (version == 2 && release < 6)) {
+		warnx("%s %d.%d is not supported", uname_buf.sysname,
+		      version, release);
+		exit(1);
 	}
 
 	if (!busid && !blockdev && !devnode) {
-		printf("Error: please specify a device using either -b, -i "
-		       "or -d\n");
-		return -1;
+		warnx("Error: please specify a device using either -b, -i or -d");
+		exit(1);
 	}
 
 	if ((busid && blockdev) || (busid && devnode) || (blockdev && devnode)) {
-		printf("Error: please specify device only once,  either -b, -i "
-		       "or -d\n");
-		return -1;
+		warnx("Error: please specify device only once,  either -b, -i or -d");
+		exit(1);
 	}
 
 	if (!print_uid && !print_extended_uid && !print_vlabel) {
-		printf("Error: no action specified (e.g. -u)\n");
-		return -1;
+		warnx("Error: no action specified (e.g. -u)");
+		exit(1);
 	}
 
-	if (((readbuf = dinfo_malloc(readbuflen)) == NULL) ||
-	    ((uidfile = dinfo_malloc(readbuflen)) == NULL))
-		return -1;
+	readbuf = dinfo_malloc(RD_BUFFER_SIZE);
+	uidfile = dinfo_malloc(RD_BUFFER_SIZE);
+	if (!(readbuf && uidfile))
+		exit(1);
 
 	/* try to read the uid attribute */
 	if (busid) {
-		sprintf(uidfile,"/sys/bus/ccw/devices/%s/uid", busid);
+		sprintf(uidfile, "/sys/bus/ccw/devices/%s/uid", busid);
 	} else if (blockdev) {
-		sprintf(uidfile,"/sys/block/%s/device/uid", blockdev);
+		sprintf(uidfile, "/sys/block/%s/device/uid", blockdev);
 	} else if (devnode) {
 		if (dinfo_get_uid_from_devnode(&uidfile, devnode) != 0)
 			goto error;
@@ -754,7 +646,7 @@ int main(int argc, char * argv[])
 	}
 
 	if (print_uid) {
-		if (dinfo_read_dasd_uid(uidfile, readbuf, readbuflen) == 0) {
+		if (dinfo_read_dasd_uid(uidfile, readbuf) == 0) {
 			/* look for the 4th '.' and cut there */
 			srchuid = readbuf - 1;
 			for (i = 0; i < 4; ++i) {
@@ -762,25 +654,23 @@ int main(int argc, char * argv[])
 				if (!srchuid)
 					break;
 			}
-			if (srchuid) {
-				srchuid[0] = '\n';
-				srchuid[1] = 0;
-			}
-			if (export) {
-				printf("ID_UID=%s",readbuf);
-			} else
-				printf("%s",readbuf);
+			if (srchuid)
+				srchuid[0] = '\0';
+			if (export)
+				printf("ID_UID=%s\n", readbuf);
+			else
+				printf("%s\n", readbuf);
 			if (!print_vlabel && !print_extended_uid)
 				goto out;
 		}
 	}
 
 	if (print_extended_uid) {
-		if (dinfo_read_dasd_uid(uidfile, readbuf, readbuflen) == 0) {
-			if (export) {
-				printf("ID_XUID=%s",readbuf);
-			} else
-				printf("%s",readbuf);
+		if (dinfo_read_dasd_uid(uidfile, readbuf) == 0) {
+			if (export)
+				printf("ID_XUID=%s\n", readbuf);
+			else
+				printf("%s\n", readbuf);
 			if (!print_vlabel)
 				goto out;
 		}
@@ -809,21 +699,23 @@ int main(int argc, char * argv[])
 			goto error;
 
 	} else if (devnode) {
-		if ((device = dinfo_malloc(readbuflen)) == NULL)
-		    return -1;
+		device = dinfo_malloc(RD_BUFFER_SIZE);
+		if (!device)
+			exit(1);
 		strcpy(device, devnode);
 	}
 
 	if (dinfo_read_dasd_vlabel(device, &vlabel, readbuf) == 0) {
-		if (export) {
-			printf("ID_SERIAL=%s\n",readbuf);
-		} else
+		if (export)
+			printf("ID_SERIAL=%s\n", readbuf);
+		else
 			printf("%s\n", readbuf);
 		goto out;
 	}
 
 error:
-	printf("Error: could not read unique DASD ID\n");
+	warnx("Error: could not read unique DASD ID");
+	rc = 1;
 
 out:
 	if (device && (busid || blockdev))
@@ -833,5 +725,5 @@ out:
 	free(device);
 	free(readbuf);
 
-	return 0;
+	exit(rc);
 }
