@@ -1,7 +1,7 @@
 /*
  * fdasd - Create or modify partitions on ECKD DASDs
  *
- * Copyright IBM Corp. 2001, 2017
+ * Copyright IBM Corp. 2001, 2018
  *
  * s390-tools is free software; you can redistribute it and/or modify
  * it under the terms of the MIT license. See LICENSE for details.
@@ -258,7 +258,7 @@ static void fdasd_check_volser(char *volser, int devno)
 {
 	int from, to;
 
-	for (from = 0, to = 0; volser[from] && from < VOLSER_LENGTH; from++)
+	for (from = 0, to = 0; from < VOLSER_LENGTH && volser[from]; from++)
 		if ((volser[from] >= 0x23 &&
 		     volser[from] <= 0x25) || /* # $ % */
 		    (volser[from] >= 0x30 &&
@@ -727,9 +727,9 @@ static int fdasd_parse_conffile(fdasd_anchor_t *anc,
 
 	memset(buffer, 0, sizeof(buffer));
 	rc = read(fd, buffer, sizeof(buffer) - 1);
+	close(fd);
 	if (rc < 0)
 		return -1;
-	close(fd);
 
 	for (i = 0; i < rc; i++)
 		buffer[i] = toupper(buffer[i]);
@@ -1257,7 +1257,7 @@ static void fdasd_write_vtoc_labels(fdasd_anchor_t *anc)
 		printf("DSCBs: ");
 
 	blk = (cchhb2blk(&anc->vlabel->vtoc, &geo) - 1) * anc->blksize;
-	if (blk <= 0)
+	if (cchhb2blk(&anc->vlabel->vtoc, &geo) == 0 || blk == 0)
 		fdasd_error(anc, vlabel_corrupted, "");
 	maxblk = blk + anc->blksize * 9; /* f4+f5+f7+3*f8+3*f9 */
 
@@ -1310,7 +1310,7 @@ static void fdasd_write_vtoc_labels(fdasd_anchor_t *anc)
 			volser[VOLSER_LENGTH] = ' ';
 			strncpy(c1, volser, VOLSER_LENGTH + 1);
 			c1 = strchr(ch, ' ');
-			strncpy(c1, s2, 31);
+			memcpy(c1, s2, 31);
 		} else {
 			if (get_part_type_by_dsname(ch, &part_info->type))
 				part_info->type = PARTITION_NATIVE;
@@ -1321,23 +1321,23 @@ static void fdasd_write_vtoc_labels(fdasd_anchor_t *anc)
 
 			setpos(anc, k, i - 1);
 
-			strncpy(ch, "LINUX.V               "
+			memcpy(ch, "LINUX.V               "
 				"                      ", 44);
 
 			strncpy(volser, anc->vlabel->volid, VOLSER_LENGTH);
 			vtoc_ebcdic_dec(volser, volser, VOLSER_LENGTH);
-			strncpy(c1, volser, VOLSER_LENGTH);
+			memcpy(c1, volser, VOLSER_LENGTH);
 
 			c1 = strchr(ch, ' ');
-			strncpy(c1, ".PART", 5);
+			memcpy(c1, ".PART", 5);
 			c1 += 5;
 
 			sprintf(dsno, "%04d.", k + 1);
-			strncpy(c1, dsno, 5);
+			memcpy(c1, dsno, 5);
 			c1 += 5;
 
 			get_part_dsname_by_type(part_info->type, &dsname);
-			strncpy(c1, dsname, strlen(dsname)); /* We don't want \0 */
+			memcpy(c1, dsname, strlen(dsname)); /* We don't want \0 */
 		}
 		vtoc_ebcdic_enc(ch, ch, 44);
 		if (anc->verbose)
@@ -1636,7 +1636,7 @@ static void fdasd_change_part_type(fdasd_anchor_t *anc)
 
 	ch = strstr(part_info->f1->DS1DSNAM, "PART") + 9;
 	if (ch != NULL)
-		strncpy(ch, str, 6);
+		memcpy(ch, str, 6);
 	vtoc_ebcdic_enc(part_info->f1->DS1DSNAM, part_info->f1->DS1DSNAM, 44);
 	anc->vtoc_changed++;
 }
@@ -1994,7 +1994,7 @@ static int fdasd_check_volume(fdasd_anchor_t *anc)
 			printf(" VOL1\n");
 
 		blk = (cchhb2blk(&vlabel->vtoc, &geo) - 1) * anc->blksize;
-		if (blk > 0) {
+		if (cchhb2blk(&vlabel->vtoc, &geo) > 0 && blk > 0) {
 			rc = fdasd_valid_vtoc_pointer(anc, blk);
 
 			if (anc->print_table && (rc < 0))
