@@ -1,27 +1,30 @@
 /*
- * File...........: u2s.c
- * Author(s)......: Gerhard Tonn <ton@de.ibm.com>
- *                  Stefan Weinhuber <wein@de.ibm.com>
  *
- * Copyright IBM Corp. 2004, 2006.
+ * Copyright IBM Corp. 2004, 2017
+ *
+ * s390-tools is free software; you can redistribute it and/or modify
+ * it under the terms of the MIT license. See LICENSE for details.
  *
  */
 
-#include <stdio.h>
-#include <unistd.h>
-#include <wait.h>
+#include <dirent.h>
 #include <errno.h>
+#include <fcntl.h>
+#include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <sys/stat.h>
-#include <sys/types.h>
-#include <fcntl.h>
 #include <sys/sysmacros.h>
-#include <dirent.h>
+#include <sys/types.h>
+#include <unistd.h>
+#include <wait.h>
 
-#include "u2s.h"
+#include "lib/u2s.h"
+#include "lib/util_libc.h"
 
 #define DEV_BUFFER_LENGTH 20
 #define PATH_BUFFER_LENGTH 256
+#define BUSIDSIZE 9
 
 #define BLOCKPATH "/sys/block/"
 #define DEVICE_LINK "device"
@@ -36,7 +39,7 @@ static int isdir(char *name) {
 
 	struct stat statbuf;
 
-	if (lstat(name, &statbuf) < 0)
+	if (stat(name, &statbuf) < 0)
  		return 0;
 	return S_ISDIR(statbuf.st_mode);
 }
@@ -56,7 +59,8 @@ static int check_directory(char *name, char *dev) {
 	char *end;
 	int fd;
 	ssize_t count;
-	int dev_attr_len, dev_parm_len, namelen;
+	int dev_attr_len, dev_parm_len;
+	unsigned int namelen;
 
 	namelen = strlen(name);
 	if ((PATH_BUFFER_LENGTH - namelen) < sizeof(DEV_ATTRIBUTE))
@@ -86,13 +90,15 @@ static int check_directory(char *name, char *dev) {
  * to hold the additional "device".
  * name: block device path in sysfs.
  * busid: buffer in which the busid string will be returned
- * returns 0 for successfull operation and -1 in case of an error.
+ * returns 0 for successful operation and -1 in case of an error.
  */
 static int extract_busid(char *name, char *busid) {
 
-	int count, namelen;
+	int count;
+	unsigned int namelen;
 	char linkbuffer[PATH_BUFFER_LENGTH];
 	char *start, *end;
+	size_t len;
 
 	namelen = strlen(name);
 	if ((PATH_BUFFER_LENGTH - namelen) < sizeof(DEVICE_LINK))
@@ -107,7 +113,10 @@ static int extract_busid(char *name, char *busid) {
 	if (!start)
 		return -1;
 	start++;
-	strcpy(busid, start);
+	len = util_strlcpy(busid, start, BUSIDSIZE);
+	if (len >= BUSIDSIZE)
+		return -1;
+
 	return 0;
 };
 
@@ -152,7 +161,7 @@ static char *append_if_directory(char *buffer, char *oldend, struct dirent *dir)
  * it's busid
  * dev: <major>:<minor> of the device
  * busid: buffer in which the busid string will be returned
- * returns 0 for successfull operation and -1 in case of an error.
+ * returns 0 for successful operation and -1 in case of an error.
  */
 static int find_busid_in_sysfs(char *dev, char *busid) {
 
@@ -210,12 +219,50 @@ static int find_busid_in_sysfs(char *dev, char *busid) {
 		return -1;
 }
 
+/*
+ * helper function that searches for a specific block device in
+ * /proc/dasd/devices and returns it's bus-ID
+ * maja, mina: <major>, <minor> of the device
+ * busid: buffer in which the bus-ID string will be returned
+ * returns 0 for successful operation and -1 in case of an error
+ * e.g. /proc/dasd/devices does not exist.
+ *
+ * An entry looks like:
+ * 0.0.XXXX(DISCIPLINE) at ( MAJ:   MIN) is dasdX       :
+ * active at blocksize: BLOCKSIZE, BLOCKS blocks, SIZE MB
+ */
+static int find_busid_in_proc(int maja, int mina, char *busid)
+{
+	FILE *filp;
+	char bus[BUSIDSIZE];
+	int majb, minb, rc;
+	size_t len;
+
+	rc = -1;
+
+	filp = fopen("/proc/dasd/devices", "r");
+	if (!filp)
+		return rc;
+	while (fscanf(filp, "%[^(] %*[^)] ) at ( %d : %d %*[^\n]\n",
+		      bus, &majb, &minb) != EOF) {
+		if ((maja == majb) && (mina == minb)) {
+			len = util_strlcpy(busid, bus, BUSIDSIZE);
+			if (len < BUSIDSIZE)
+				rc = 0;
+			break;
+		}
+	}
+
+	fclose(filp);
+	return rc;
+}
+
 /*
  * Return the busid of a given device node.
  * Works only for block devices.
  * devicenode: path to the device node
  * busid: buffer in which the busid string will be returned
- * returns 0 for successfull operation and -1 in case of an error.
+ * returns 0 for successful operation and -1 in case of an error.
  */
 int u2s_getbusid(char *devicenode, char *busid)
 {
@@ -234,9 +281,54 @@ int u2s_getbusid(char *devicenode, char *busid)
                 return -1;
         maj = major(stat_buf.st_rdev);
         min = minor(stat_buf.st_rdev);
-        snprintf(dev_string,DEV_BUFFER_LENGTH,"%u:%u", maj, min);
 
-	rc = find_busid_in_sysfs(dev_string, busid);
+	rc = find_busid_in_proc(maj, min, busid);
+	if (rc) {
+		snprintf(dev_string, DEV_BUFFER_LENGTH, "%u:%u", maj, min);
+		rc = find_busid_in_sysfs(dev_string, busid);
+	}
 
 	return rc;
 }
+
+/*
+ * Attempts to find the sysfs entry for the given busid and reads
+ * the contents of a specified attribute to the buffer
+ */
+int u2s_read_attribute(char *busid, char *attribute, char *buffer,
+		       size_t count)
+{
+	char path[100];
+	int rc, fd;
+	ssize_t rcount;
+
+	rc = 0;
+	snprintf(path, sizeof(path), "/sys/bus/ccw/devices/%s/%s",
+		 busid, attribute);
+	fd = open(path, O_RDONLY);
+	if (fd < 0)
+		return errno;
+	rcount = read(fd, buffer, count);
+	if (rcount < 0)
+		rc = errno;
+	close(fd);
+	return rc;
+}
+
+int u2s_get_host_access_count(char *devicenode)
+{
+	char busid[BUSIDSIZE];
+	unsigned long value;
+	char buffer[10];
+	char *endp;
+
+	u2s_getbusid(devicenode, busid);
+	u2s_read_attribute(busid, "host_access_count", buffer, sizeof(buffer));
+
+	value = strtoul(buffer, &endp, 0);
+
+	if (endp == buffer)
+		return -EINVAL;
+
+	return value;
+}
