@@ -48,6 +48,7 @@
  * Definitions for the CCA library
  */
 #define CCA_LIBRARY_NAME	"libcsulcca.so"
+#define CCA_WEB_PAGE		"http://www.ibm.com/security/cryptocards"
 
 #define DEFAULT_KEYBITS 256
 
@@ -71,16 +72,20 @@ int load_cca_library(void **lib_csulcca, t_CSNBKTC *dll_CSNBKTC, bool verbose)
 	/* Load the CCA library */
 	*lib_csulcca = dlopen(CCA_LIBRARY_NAME, RTLD_GLOBAL | RTLD_NOW);
 	if (*lib_csulcca == NULL) {
-		warnx("%s\nEnsure that the IBM CCA Host Libraries and "
-		      "Tools are installed properly", dlerror());
+		pr_verbose(verbose, "%s", dlerror());
+		warnx("The command requires the IBM CCA Host Libraries and "
+		      "Tools.\nFor the supported environments and downloads, "
+		      "see:\n%s", CCA_WEB_PAGE);
 		return  -ELIBACC;
 	}
 
 	/* Get the Key Token Change function */
 	*dll_CSNBKTC = (t_CSNBKTC)dlsym(*lib_csulcca, "CSNBKTC");
 	if (*dll_CSNBKTC == NULL) {
-		warnx("%s\nEnsure that the IBM CCA Host Libraries and "
-		      "Tools are installed properly", dlerror());
+		pr_verbose(verbose, "%s", dlerror());
+		warnx("The command requires the IBM CCA Host Libraries and "
+		      "Tools.\nFor the supported environments and downloads, "
+		      "see:\n%s", CCA_WEB_PAGE);
 		dlclose(*lib_csulcca);
 		*lib_csulcca = NULL;
 		return -ELIBACC;
@@ -158,8 +163,8 @@ u8 *read_secure_key(const char *keyfile, size_t *secure_key_size,
 
 	buf = util_malloc(size);
 	count = fread(buf, 1, size, fp);
-	if (count <= 0) {
-		msg = feof(fp) ? "File is too small" : strerror(errno);
+	if (count != size) {
+		msg = ferror(fp) ? strerror(errno) : "File is too small";
 		warnx("File '%s': %s", keyfile, msg);
 		free(buf);
 		buf = NULL;
@@ -206,7 +211,7 @@ int write_secure_key(const char *keyfile, const u8 *secure_key,
 	}
 
 	count = fwrite(secure_key, 1, secure_key_size, fp);
-	if (count <= 0) {
+	if (count != secure_key_size) {
 		warnx("File '%s': %s", keyfile, strerror(errno));
 		fclose(fp);
 		return -EIO;
@@ -296,8 +301,8 @@ static u8 *read_clear_key(const char *keyfile, size_t keybits, bool xts,
 
 	buf = util_malloc(size);
 	count = fread(buf, 1, size, fp);
-	if (count <= 0) {
-		msg = feof(fp) ? "File is too small" : strerror(errno);
+	if (count != size) {
+		msg = ferror(fp) ? strerror(errno) : "File is too small";
 		warnx("File '%s': %s", keyfile, msg);
 		free(buf);
 		buf = NULL;
